@@ -1,11 +1,11 @@
 """
-Components for Tango integration with `🤗 Datasets <https://huggingface.co/docs/datasets/>`_.
-
 .. important::
     To use this integration you should install ``tango`` with the "datasets" extra
     (e.g. ``pip install tango[datasets]``) or just install the ``datasets`` library after the fact
     (e.g. ``pip install datasets``).
 
+Components for Tango integration with `🤗 Datasets <https://huggingface.co/docs/datasets/>`_.
+
 Example: loading and combining
 ------------------------------
 
@@ -23,13 +23,12 @@ You could run this with:
 """
 
 
-from typing import Union, TypeVar, List, Optional, Any
+from typing import Any, List, Optional, TypeVar, Union, overload
 
 import datasets as ds
 
+from tango.common.dataset_dict import DatasetDict, DatasetDictBase, IterableDatasetDict
 from tango.step import Step
-from tango.common.dataset_dict import DatasetDict
-
 
 __all__ = [
     "LoadDataset",
@@ -39,18 +38,37 @@ __all__ = [
 ]
 
 
+@overload
 def convert_to_tango_dataset_dict(hf_dataset_dict: ds.DatasetDict) -> DatasetDict:
+    ...
+
+
+@overload
+def convert_to_tango_dataset_dict(hf_dataset_dict: ds.IterableDatasetDict) -> IterableDatasetDict:
+    ...
+
+
+def convert_to_tango_dataset_dict(hf_dataset_dict):
     """
     A helper function that can be used to convert a HuggingFace :class:`~datasets.DatasetDict`
-    into a native Tango :class:`~tango.common.dataset_dict.DatasetDict`.
+    or :class:`~datasets.IterableDatasetDict` into a native Tango
+    :class:`~tango.common.DatasetDict` or :class:`~tango.common.IterableDatasetDict`.
 
     This is important to do when your dataset dict is input to another step for caching
     reasons.
     """
-    fingerprint = ""
-    for key, dataset in sorted(hf_dataset_dict.items(), key=lambda x: x[0]):
-        fingerprint += f"{key}-{dataset._fingerprint}-"
-    return DatasetDict(splits=hf_dataset_dict, fingerprint=fingerprint)
+    if isinstance(hf_dataset_dict, ds.IterableDatasetDict):
+        # There is no way to know when an iterable dataset will yeild different elements,
+        # so we generate a random fingerprint to use.
+        from uuid import uuid4
+
+        fingerprint = str(uuid4())
+        return IterableDatasetDict(splits=hf_dataset_dict, fingerprint=fingerprint)
+    else:
+        fingerprint = ""
+        for key, dataset in sorted(hf_dataset_dict.items(), key=lambda x: x[0]):
+            fingerprint += f"{key}-{dataset._fingerprint}-"
+        return DatasetDict(splits=hf_dataset_dict, fingerprint=fingerprint)
 
 
 @Step.register("datasets::load")
@@ -103,12 +121,11 @@ class InterleaveDatasets(Step):
         datasets: List[DatasetType],
         probabilities: Optional[List[float]] = None,
         seed: Optional[int] = None,
-        **kwargs,
     ) -> DatasetType:
         """
         Interleave the list of datasets.
         """
-        return ds.interleave_datasets(datasets, probabilities=probabilities, seed=seed, **kwargs)
+        return ds.interleave_datasets(datasets, probabilities=probabilities, seed=seed)
 
 
 @Step.register("datasets::concatenate")
