@@ -104,7 +104,7 @@ would produce the following output:
     ● Starting run for "data"...
     ✓ Finished run for "data"
     ● Starting run for "train"...
-    Loading best weights from state_worker0_step100.pt
+    Loading best weights from state_worker0_best.pt
     ✓ Finished run for "train"
 
 Tips
@@ -138,6 +138,7 @@ __all__ = [
     "Sampler",
     "ConcatTensorDictsCollator",
     "TrainCallback",
+    "TrainConfig",
     "StopEarlyCallback",
     "StopEarly",
 ]
@@ -149,3 +150,4 @@ from .model import Model
 from .optim import LRScheduler, Optimizer
 from .train import TorchTrainStep
 from .train_callback import StopEarlyCallback, TrainCallback
+from .train_config import TrainConfig
